@@ -5,7 +5,8 @@ import os
 
 # dict mapping arch -> ( multicompat, best personality, biarch personality )
 multilibArches = { "x86_64":  ( "athlon", "x86_64", "athlon" ),
-                   "sparc64": ( "sparc", "sparc", "sparc64" ),
+                   "sparc64v": ( "sparc", "sparcv9v", "sparc64v" ),
+                   "sparc64": ( "sparc", "sparcv9", "sparc64" ),
                    "ppc64":   ( "ppc", "ppc", "ppc64" ),
                    "s390x":   ( "s390", "s390x", "s390" ),
                    "ia64":    ( "i686", "ia64", "i686" )
@@ -38,7 +39,10 @@ arches = {
     "s390": "noarch",
     
     # sparc
+    "sparc64v": "sparc64",
+    "sparc64v": "sparcv9v",
     "sparc64": "sparcv9",
+    "sparcv9v": "sparcv9",
     "sparcv9": "sparcv8",
     "sparcv8": "sparc",
     "sparc": "noarch",
@@ -67,7 +71,7 @@ def score(arch):
 def isMultiLibArch(arch=None):
     """returns true if arch is a multilib arch, false if not"""
     if arch is None:
-        arch = getCanonArch()
+        arch = canonArch
 
     if not arches.has_key(arch): # or we could check if it is noarch
         return 0
@@ -87,11 +91,18 @@ def getBestArchFromList(archlist, myarch=None):
     """
     
     if myarch is None:
-        myarch = getCanonArch()
-
+        myarch = canonArch
+    
+    mybestarch = getBestArch(myarch)
+    
     if len(archlist) == 0:
         return None
-    
+    bestarch = getBestArch(myarch)
+    if bestarch != myarch:
+        bestarchchoice = getBestArchFromList(archlist, bestarch)
+        if bestarchchoice != None and bestarchchoice != "noarch":
+            return bestarchchoice
+            
     thisarch = archlist[0]
     for arch in archlist[1:]:
         val1 = archDifference(myarch, thisarch)
@@ -119,12 +130,17 @@ def getBestArchFromList(archlist, myarch=None):
 def getArchList(thisarch=None):
     # this returns a list of archs that are compatible with arch given
     if not thisarch:
-        thisarch = getCanonArch()
+        thisarch = canonArch
     
     archlist = [thisarch]
     while arches.has_key(thisarch):
         thisarch = arches[thisarch]
         archlist.append(thisarch)
+
+    # hack hack hack
+    # sparc64v is also sparc64 compat
+    if archlist[0] == "sparc64v":
+        archlist.insert(1,"sparc64")
     
     return archlist
     
@@ -170,6 +186,29 @@ def getCanonPPCArch(arch):
         return "ppc64iseries"
     return arch
 
+def getCanonSPARCArch(arch):
+    # Deal with sun4v, sun4u, sun4m cases
+    f = open("/proc/cpuinfo", "r")
+    lines = f.readlines()
+    f.close()
+    for line in lines:
+        if line.startswith("type"):
+            SPARCtype = line.split(':')[1]
+            break
+    if SPARCtype.find("sun4v") != -1:
+        if arch.startswith("sparc64"):
+            return "sparc64v"
+        else:
+            return "sparcv9v"
+    if SPARCtype.find("sun4u") != -1:
+        if arch.startswith("sparc64"):
+            return "sparc64"
+        else:
+            return "sparcv9"
+    if SPARCtype.find("sun4m") != -1:
+        return "sparcv8"
+    return arch
+
 def getCanonX86_64Arch(arch):
     if arch != "x86_64":
         return arch
@@ -209,13 +248,17 @@ def getCanonArch(skipRpmPlatform = 0):
 
     if arch.startswith("ppc"):
         return getCanonPPCArch(arch)
+    if arch.startswith("sparc"):
+        return getCanonSPARCArch(arch)
     if arch == "x86_64":
         return getCanonX86_64Arch(arch)
 
     return arch
 
+canonArch = getCanonArch()
+
 # this gets you the "compat" arch of a biarch pair
-def getMultiArchInfo(arch = getCanonArch()):
+def getMultiArchInfo(arch = canonArch):
     if multilibArches.has_key(arch):
         return multilibArches[arch]
     if arches.has_key(arch) and arches[arch] != "noarch":
@@ -225,14 +268,17 @@ def getMultiArchInfo(arch = getCanonArch()):
 # get the best usual userspace arch for the arch we're on.  this is
 # our arch unless we're on an arch that uses the secondary as its
 # userspace (eg ppc64, sparc64)
-def getBestArch():
-    arch = canonArch
+def getBestArch(myarch=None):
+    if myarch:
+        arch = myarch
+    else:
+        arch = canonArch
 
-    if arch == "sparc64":
-        arch = "sparc"
+    if arch.startswith("sparc64"):
+        arch = multilibArches[arch][1]
 
     if arch.startswith("ppc64"):
-        arch = "ppc"
+        arch = 'ppc'
 
     return arch
 
@@ -242,12 +288,12 @@ def getBaseArch(myarch=None):
        a key in the multilibArches."""
 
     if not myarch:
-        myarch = getCanonArch()
+        myarch = canonArch
 
     if not arches.has_key(myarch): # this is dumb, but <shrug>
         return myarch
 
-    if myarch == "sparc64":
+    if myarch.startswith("sparc64"):
         return "sparc"
     elif myarch.startswith("ppc64"):
         return "ppc"
@@ -267,4 +313,3 @@ def getBaseArch(myarch=None):
     
         return basearch
         
-canonArch = getCanonArch()
